# -*- coding: utf-8 -*-
# vim: tabstop=4 shiftwidth=4 softtabstop=4 expandtab

from maud.cwindow_func import window_func, window_func_scalar

import numpy as np
from numpy import ma

cimport numpy as np
from libc.math cimport cos

#from fluid.common.distance import distance
#from fluid.common.distance import find_closer_then
#from fluid.cdistance import distance
from fluid.cdistance import find_closer_then
from fluid.cdistance import _distance_scalar

np.import_array()

DTYPE = np.float
ctypedef np.float_t DTYPE_t


def window_1Dmean(data, double l, t=None, method='hann', axis=0, parallel=True):
    """ A moving window mean filter, not necessarily a regular grid.

        1D means that the filter is applied to along only one
          of the dimensions, but in the whole array. For example
          in a 3D array, each latXlon point is filtered along the
          time.

        It's not optimized for a regular grid.

        t is the scale of the choosed axis

        l is the size of the filter.
    """
    assert axis <= data.ndim, "Invalid axis!"

    # If necessary, move the axis to be filtered for the first axis
    if axis != 0:
        data_smooth = window_1Dmean(data.swapaxes(0,axis),
            l = l,
            t = t,
            method = method,
            axis=0,
            parallel = parallel)

        return data_smooth.swapaxes(0,axis)
    # Bellow here, the filter will be always applied on axis=0

    # If t is not given, creates a regularly spaced t
    if t == None:
        print "The scale along the choosed axis weren't defined. I'll consider a constant sequence."
        t = np.arange(data.shape[axis])

    # t must has the same shape of data along axis
    assert t.shape == (data.shape[axis],)

    # ----
    winfunc = window_func(method)

    data_smooth = ma.masked_all(data.shape)

    cdef int i

    if data.ndim==1:
        # It's faster than getmaskarray
        (I,) = np.nonzero(np.isfinite(data))
        for i in I:
            dt = t-t[i]
            ind = np.nonzero((np.absolute(dt)<l))
            w = winfunc(dt[ind],l)
            data_smooth[i] = (data[ind]*w).sum()/(w.sum())

    else:
        I = data.shape[1]
        for i in range(I):
            data_smooth[:,i] = window_1Dmean(data[:,i],
                    l = l,
                    t = t,
                    method = method,
                    axis = 0,
                    parallel=parallel)

    return data_smooth

def window_mean_2D_latlon(Lat, Lon, data, l, method='hamming', interp=False):
    """
        Right now I'm doing for Masked Arrays only.
        data should be a dictionary with 2D arrays

        Input:
          - Lat: 2D array with latitudes
          - Lon: 2D array with longitudes
          - data: There are two possibilities, it can be an
            array of at least 2D, or a dictionary where each
            element is an array of at least 2D.
          - l: window filter size, in meters
          - method: weight function type

        Output:

        !!!ATENTION!!!
        - Might be a good idea to eliminate the dependence on
          fluid.

	- I can't get the shape of an np.ndarray[DTYPE_t, ndim=2],
	    so I need to use data2, a stupid solution
    """

    #if type(data) == dict:
    #    output = {}
    #    for k in data.keys():
    #        output[k] = window_mean_2D_latlon(Lat, Lon, data[k], l, method)
    #    return output

    assert (Lat.ndim == 2) & (Lon.ndim == 2), "Lat and Lon must be 2D array"
    assert data.ndim == 2, "Sorry, for now I'm only handeling 2D arrays"

    if type(data) == np.ndarray:
        return _window_mean_2D_latlon(Lat, Lon, data, l, method)

    elif type(data) == ma.MaskedArray:
        if (data.mask == False).all():
            data_smooth = _window_mean_2D_latlon(Lat, Lon, data.data, l, method)
            return ma.array(data_smooth) 
        else:
            data_smooth, mask = _window_mean_2D_latlon_masked(Lat, Lon, data.data, data.mask.astype('int8'), l, method, interp)
            return ma.masked_array(data_smooth, mask)

    #if interp == True:
    #    I, J = data.shape
    #    I, J = np.meshgrid(range(I), range(J))
    #    I = I.reshape(-1); J = J.reshape(-1)
    #else:
    #    I, J = np.nonzero(np.isfinite(data))

#        #ind, r = find_closer_then(Lat, Lon, Lat[i,j], Lon[i,j], llimit=l)
#        #if len(r) > 0:
#        #    w = weight_func(r, l)
#        #    tmp = data[ind]*w
#        #    wsum = w[np.nonzero(tmp)].sum()
#        #    if wsum != 0:
#        #        data_smooth[i,j] = (tmp).sum()/wsum
#	#elif data.ndim == 3:
#	#    for k in range(data.shape[0]):
#	#        data_smooth[k,i,j] = (data[k][ind]*w).sum()/w[good].sum()
#	#    else:
#	#        data_smooth[k,i,j] = (data[k][ind]*w).sum()/w[good].sum()
#    return data_out
    #return data_smooth

def _window_mean_2D_latlon(np.ndarray[DTYPE_t, ndim=2] Lat, np.ndarray[DTYPE_t, ndim=2] Lon, np.ndarray[DTYPE_t, ndim=2] data, l, method='hamming'):
#    """
#    """
    weight_func = window_func_scalar(method)

    cdef unsigned int i, ii, j, jj
    cdef double r, w
    cdef double W, D
    cdef unsigned int I = data.shape[0]
    cdef unsigned int J = data.shape[1]
    cdef np.ndarray[DTYPE_t, ndim=2] data_smooth = np.empty((I,J))

    for i in range(I):
        for j in range(J):
            W = 0
            D = 0
            for ii in range(I):
                for jj in range(J):
                    r = _distance_scalar(Lat[i,j], Lon[i,j], Lat[ii,jj], Lon[ii,jj])
                    if r <= l:
                        w = weight_func(r, l)
                        if w != 0:
                            D += data[ii, jj] * w
                            W += w
            if W != 0:
                data_smooth[i, j] = D/W

    return data_smooth

def _window_mean_2D_latlon_masked(np.ndarray[DTYPE_t, ndim=2] Lat, np.ndarray[DTYPE_t, ndim=2] Lon, np.ndarray[DTYPE_t, ndim=2] data, np.ndarray[np.int8_t, ndim=2] mask, l, method, interp):
#    """
#    """
    weight_func = window_func_scalar(method)

    cdef unsigned int i, ii, j, jj
    cdef double r, w
    cdef double W, D
    cdef unsigned int I = data.shape[0]
    cdef unsigned int J = data.shape[1]
    cdef np.ndarray[DTYPE_t, ndim=2] data_smooth = np.empty((I,J))
    cdef np.ndarray[np.int8_t, ndim=2] mask_smooth = np.ones((I,J), dtype=np.int8)

    for i in range(I):
        for j in range(J):
            if (interp == True) or (mask[i, j] == 0):
                W = 0
                D = 0
                for ii in range(I):
                    for jj in range(J):
                        if mask[ii, jj] == 0:
                            r = _distance_scalar(Lat[i,j], Lon[i,j], Lat[ii,jj], Lon[ii,jj])
                            if r <= l:
                                w = weight_func(r, l)
                                if w != 0:
                                    D += data[ii, jj] * w
                                    W += w
                if W != 0:
                    data_smooth[i, j] = D/W
                    mask_smooth[i, j] = 0

    return data_smooth, mask_smooth




# Gui, 27-06-2012
# Just an idea. Create a class of filtered data. A fake Masked Array object
#   which gives the output on demand, filtered.

import multiprocessing

import numpy as N
import numpy
import numpy as np
from numpy import ma

try:
    from fluid.cdistance import distance
    from fluid.cdistance import find_closer_then
except:
    from fluid.common.distance import distance
    from fluid.common.distance import find_closer_then

#try:
#    from maud.cwindow_func import window_func
#except:
#    from window_func import window_func
from window_func import window_func


"""
"""


def window_mean_2D_latlon(Lat, Lon, data, l, method='hamming', interp='False'):
    """
        Right now I'm doing for Masked Arrays only.
        data should be a dictionary with 2D arrays

        Input:
          - Lat: 2D array with latitudes
          - Lon: 2D array with longitudes
          - data: There are two possibilities, it can be an 
            array of at least 2D, or a dictionary where each
            element is an array of at least 2D.
          - l: window filter size, in meters
          - method: weight function type
          - interp: if False, estimate only for the gridponits
              of valid data, i.e. data.mask = output.mask. If
              True, estimate for all gridpoints that has at
              least one valida data point inside the l/2 radius 

        Output:

        !!!ATENTION!!!
        - Might be a good idea to eliminate the dependence on
          fluid.
    """
    if type(data) == dict:
        output = {}
        for k in data.keys():
            output[k] = window_mean_2D_latlon(Lat, Lon, data[k], l, method)
        return output

    assert data.ndim == 2, "Sorry, for now I'm only handeling 2D arrays"

    weight_func = window_func(method)

    data_smooth = ma.masked_all(data.shape)

    if interp == True:
        I, J = data.shape
        I, J = np.meshgrid(range(I), range(J))
        I = I.reshape(-1); J = J.reshape(-1)
    else:
        I, J = np.nonzero(np.isfinite(data))

    for i, j in zip(I, J):
            ind, r = find_closer_then(Lat, Lon, Lat[i,j], Lon[i,j], llimit=l)
            if len(r) > 0:
                w = weight_func(r, l)
                if data.ndim == 2:
                        #good = np.nonzero(data[ind])
                        tmp = data[ind]*w
                        wsum = w[np.nonzero(tmp)].sum()
                        if wsum != 0:
                            data_smooth[i,j] = (tmp).sum()/wsum
                #elif data.ndim == 3:
                #    for k in range(data.shape[0]):
                #        data_smooth[k,i,j] = (data[k][ind]*w).sum()/w[good].sum()
                #    else:
                #        data_smooth[k,i,j] = (data[k][ind]*w).sum()/w[good].sum()
    return data_smooth

# ==== Bellow here, I need to do some serious work on ====

def window_mean_2D(x, y, z, l, method='hamming'):
    """
    """
    if method == 'hamming':
        weight_func = _weight_hamming_2D

    if len(z.shape) < 2:
        print "At least 2D"

    output = ma.masked_all(z.shape)
    if len(z.shape) > 2:
        for i in range(z.shape[0]):
            output[i] = window_mean_2D(x, y, z[i], method)

    elif len(z.shape) == 2:
        I,J = z.shape
        for i in range(I):
            for j in range(J):
	        w = weight_func((x-x[i,j]), (y-y[i,j]), l)
	        output[i,j] = (z*w).sum()/(w.sum())
        return output



def window_mean(y,x=None,x_out=None,method="rectangular",boxsize=None):
    """Windowed means along 1-D array

    Input:
        - x [0,1,2,3,...] =>
        - x_out [x] =>
        - method [rectangular]:
            + rectangular => All data in window have same weight
        - boxsize [mean space] =>
    Output:

    Apply windowed means on a 1-D data array. Selecting adequate x_out
    and boxsize could define boxmeans or smooth filters. Method defines
    the weight method.

    An important point of this function is the ability to work with 
    unhomogenious spaced samples. Data ([1,2,3]) colected at [1,2,4] 
    times would be different if was collected at [1,2,3].
    """
    if(x==None):
        x=N.arange(N.size(y))

    if(x_out==None):
        x_out=x

    #y_out = N.zeros(N.size(x_out),N.float)
    y_out = ma.masked_all(x_out.shape)

    if(boxsize==None):
        # !!! Improve it! A better way than *1. ?!
        boxsize =(max(x)-min(x))/(N.size(x_out)*1.)

    half_boxsize = boxsize/2.

    #for x_i in x_out:
    for i in range(N.size(x_out)):
        x_i = x_out[i]


        # Higher window limit
        hi_limit = x_i+half_boxsize
        # Lower window limit
        lo_limit = x_i-half_boxsize
        # index of values inside window
        index = N.less_equal(x,hi_limit)*N.greater_equal(x,lo_limit)

        # !!! INSERT some type of check for minimum number of samples to be considered

        # x values on window around x_i
        x_tmp = N.compress(index,x)-x_i
        # y values on window
        y_tmp = N.compress(index,y)

        # weights in window according to x position
        weight = window_weight(x_tmp,boxsize,method)

        y_out[i] = N.sum(y_tmp*weight)

    return y_out

# To improve, the right to way to implement these filters are to define the halfpower cutoff, instead of an l dimension. Then the l dimension is defined on the function according to the weightning system for the adequate l.

def _convolve(x, dt, l, winfunc):
    w = winfunc(dt, l)
    y = (x*w).sum()/(w[x.mask==False].sum())
    return y

def window_1Dbandpass(data, lshorterpass, llongerpass, t=None, method='hann', axis=0, parallel=True):
    """

        Input:
            lshorterpass: shorter wavelenghts are preserved
            llongerpass: longer wavelengths are preserved
    """
    if axis > len(data.shape):
        print "The data array don't contain so many dimensions. Choose another axis"
	return

    if t == None:
        print "The scale along the choosed axis weren't defined. I'll consider a constant sequence."
	t = numpy.arange(data.shape[axis])

    elif t.shape != (data.shape[axis],):
        print "The scale variable t don't have the same size of the choosed axis of the data array"
        return 
    # ----
    #data_smooth = ma.masked_all(data.shape)
    data_smooth = window_1Dmean(data,
                        t = t,
                        l = llongerpass,
                        axis = axis,
                        parallel = False)

    data_smooth = data_smooth - window_1Dmean(data_smooth,
                        t = t,
                        l = lshorterpass,
                        axis = axis,
                        parallel=False)

    return data_smooth


def window_1Dmean(data, l, t=None, method='hann', axis=0, parallel=True):
    """ A moving window mean filter, not necessarily a regular grid.

        1D means that the filter is applied to along only one
          of the dimensions, but in the whole array. For example
          in a 3D array, each latXlon point is filtered along the
          time.

        It's not optimized for a regular grid.

	t is the scale of the choosed axis

        l is the size of the filter.
    """
    assert axis <= data.ndim, "Invalid axis!"

    if axis != 0:
        data_smooth = window_1Dmean(data.swapaxes(0,axis),
            l = l,
            t = t,
            method = method,
            axis=0,
            parallel = parallel)

        return data_smooth.swapaxes(0,axis)

    if t == None:
        print "The scale along the choosed axis weren't defined. I'll consider a constant sequence."
	t = numpy.arange(data.shape[axis])

    elif t.shape != (data.shape[axis],):
        print "Invalid size of t."
        return 
    # ----
    winfunc = window_func(method)

    data_smooth = ma.masked_all(data.shape)

    if data.ndim==1:
        # It's faster than getmaskarray
        (I,) = np.nonzero(np.isfinite(data))
	for i in I:
            dt = t-t[i]
            ind = numpy.nonzero((numpy.absolute(dt)<l))
            w = winfunc(dt[ind],l)
            data_smooth[i] = (data[ind]*w).sum()/(w.sum())

    else:
        I = data.shape[1]
        for i in range(I):
            data_smooth[:,i] = window_1Dmean(data[:,i], 
                    l = l,
                    t = t,
                    method = method, 
                    axis = 0, 
                    parallel=parallel)

    return data_smooth

#    elif len(data.shape)==2:
#        (I,J) = data.shape
#        if parallel == True:
#            nprocesses = 2*multiprocessing.cpu_count()
#            #logger.info("I'll work with %s parallel processes" % nprocesses)
#            filters_pool = multiprocessing.Pool(nprocesses)
#            results = []
#            for j in range(J):
#                results.append(filters_pool.apply_async(window_1Dmean, (data[:,j], l, t, method, axis, parallel)))
#            filters_pool.close()
#            for n, r in enumerate(results):
#                data_smooth[:,n] = r.get()
#        else:
#            for j in range(J):
#                data_smooth[:,j] = window_1Dmean(data[:,j], l, t, method, axis)


def window_1Dmean_grid(data, l, method='hann', axis=0, parallel=False):
    """ A moving window mean filter applied to a regular grid.

        1D means that the filter is applied to along only one
          of the dimensions, but in the whole array. For example
          in a 3D array, each latXlon point is filtered along the
          time.

        The other types of filter consider the scales of each
          dimension. On this case it's considered a regular
          grid, so the filter can be based on the number of
          elements, and so be much optimized.

        l is in number of cells around the point being evaluated.
    """
    assert axis <= data.ndim, "Invalid axis!"

    if axis != 0:
        data_smooth = window_1Dmean_grid(data.swapaxes(0, axis), 
                l = l, 
                method = method, 
                axis = 0, 
                parallel = parallel)

        return data_smooth.swapaxes(0, axis)

    winfunc = window_func(method)
    r = np.arange(-np.floor(l/2),np.floor(l/2)+1)
    w = winfunc(r, l)

    data_smooth = ma.masked_all(data.shape)

    I = data.shape[0]
    norm = np.convolve(np.ones(I), w ,mode='same')
    if len(data.shape)==1:
	norm=numpy.convolve(numpy.ones(I),w,mode='same')
	data_smooth[:] = numpy.convolve(data[:],w,mode='same')/norm

    elif len(data.shape) == 2:
        I, J = data.shape
        for j in range(J):
            data_smooth[:,j] = np.convolve(data[:,j], w, mode='same')/norm

    elif len(data.shape) >2:
        I, J = data.shape[:2]
        for j in range(J):
                data_smooth[:,j] = window_1Dmean_grid(data[:,j], 
                        l = l, 
                        method = method, 
                        axis=0, 
                        parallel = parallel)

    try:
        data_smooth.mask = data.mask
    except:
        pass

    return data_smooth


# ----
def get_halfpower_period(data, filtered):
    """ Returns the gain per frequency
    """
    nt,ni,nj = data.shape
    gain = ma.masked_all((nt,ni,nj))
    for i in range(ni):
        for j in range(nj):
	    if ~filtered[:,i,j].mask.all():
	        gain[:,i,j] = numpy.absolute(numpy.fft.fft(filtered[:,i,j]-filtered[:,i,j].mean())) / numpy.absolute(numpy.fft.fft(data[:,i,j]-data[:,i,j].mean()))
    gain_median = ma.masked_all(nt)
    gain_25 = ma.masked_all(nt)
    gain_75 = ma.masked_all(nt)
    # Run for each frequency, which are in the same number of timesteps
    from scipy.stats import scoreatpercentile
    for t in range(nt):
        #gain_median[t] = numpy.median(gain[t,:,:].compressed()[numpy.isfinite(gain[t,:,:].compressed())])
        tmp = gain[t,:,:].compressed()[numpy.isfinite(gain[t,:,:].compressed())]
        gain_median[t] = scoreatpercentile(tmp,50)
        gain_25[t] = scoreatpercentile(tmp,25)
        gain_75[t] = scoreatpercentile(tmp,75)

    freq=numpy.fft.fftfreq(nt)/dt.days

    #from scipy.interpolate import UnivariateSpline
    #s = UnivariateSpline(gain_median[numpy.ceil(nt/2.):], -freq[numpy.ceil(nt/2.):], s=1)
    #xs = -freq[numpy.ceil(nt/2.):]
    #ys = s(xs)

    import rpy2.robjects as robjects
    smooth = robjects.r['smooth.spline'](robjects.FloatVector(gain_median[numpy.ceil(nt/2.):]),robjects.FloatVector(-freq[numpy.ceil(nt/2.):]),spar=.4)
    #smooth = robjects.r['smooth.spline'](robjects.FloatVector(-freq[numpy.ceil(nt/2.):]),robjects.FloatVector(gain_median[numpy.ceil(nt/2.):]),spar=.4)
    s_interp = robjects.r['predict'](smooth,x=0.5)
    halfpower_period = 1./s_interp.rx2['y'][0]

    #smooth = robjects.r['smooth.spline'](robjects.FloatVector(-freq[numpy.ceil(nt/2.):]),robjects.FloatVector(gain_median[numpy.ceil(nt/2.):]),spar=.4)
    #s_interp = robjects.r['predict'](smooth, x = robjects.FloatVector(-freq[numpy.ceil(nt/2.):]))

    #print "Filter half window size: %s" % l
    #print "Half Power Period: %s" % halfpower_period
    #self.halfpower_period = halfpower_period

    return halfpower_period



#pylab.plot(-freq[numpy.ceil(nt/2.):],  gain_median[numpy.ceil(nt/2.):]  )
#pylab.plot(1./halfpower_period, 0.5,'rx')
#pylab.show()

# In the future implement a check to show the average effect in the
#   spectrum. I.e. estimate fft before and after and compare the difference
#I,J,K = ssh.shape
##fft_input = ma.masked_all((I,J,K))
##fft_output = ma.masked_all((I,J,K))
#tmp = ma.masked_all((I,J,K))
#tmp2 = ma.masked_all(I)
#for j in range(J):
#    for k in range(K):
#        if ssh[:,j,k].mask.any()==False:
#            #fft_input[:,j,k] = numpy.fft.fft(ssh[:,j,k])
#            fft_input = numpy.absolute(numpy.fft.fft(ssh[:,j,k]))
#            #fft_output[:,j,k] = numpy.fft.fft(output[:,j,k])
#            fft_output = numpy.absolute(numpy.fft.fft(output[:,j,k]))
#            tmp[:,j,k] = (fft_input-fft_output)/fft_input
#for i in range(I):
#    tmp2[i] = tmp[i].mean()







#ssh_smooth=window_mean_1D_grid(data['ssh'],l=7)
#pylab.plot(data['ssh'][:,j,k])
#pylab.plot(data_smooth[:,j,k],'r')
#pylab.show()
#
#output=ma.masked_all(x.shape)
#for i in range(l,I-1):
#    output[i]=
#
#x=numpy.arange(10)
#w = window_mean._weight_hann(numpy.arange(2*l+1)-l,l)
#w=numpy.array([1,2,1])
#y=numpy.convolve(x,w,mode='same')
#y.shape
#x
#w
#y




from numpy import pi

#def _weight_hamming_2D(x,y,l):
#    """
#
#    Check it!
#    """
#    r=(x**2+y**2)**0.5
#    w=0.54 - 0.46 * numpy.cos(pi*r/l)
#    w[r>l]=0
#    return w



#function w = window(N,wt)
#%
#%  w = window(N,wt)
#%
#%  generate a window function
#%
#%  N = length of desired window
#%  wt = window type desired
#%       'rect' = rectangular        'tria' = triangular (Bartlett)
#%       'hann' = Hanning            'hamm'  = Hamming
#%       'blac' = Blackman
#%
#%  w = row vector containing samples of the desired window
#nn = N-1;
#pn = 2*pi*(0:nn)/nn;
#if wt(1,1:4) == 'rect',
#                        w = ones(1,N);
#elseif wt(1,1:4) == 'tria',
#                        m = nn/2;
#                        w = (0:m)/m;
#                        w = [w w(ceil(m):-1:1)];
#elseif wt(1,1:4) == 'hann',
#                        w = 0.5*(1 - cos(pn));
#elseif wt(1,1:4) == 'hamm',
#                        w = .54 - .46*cos(pn);
#elseif wt(1,1:4) == 'blac',
#                        w = .42 -.5*cos(pn) + .08*cos(2*pn);
#else
#                        disp('Incorrect Window type requested')
#end





#!/usr/bin/env python
# -*- coding: Latin-1 -*-
# vim: tabstop=4 shiftwidth=4 expandtab

""" Filter uneven spaced data.
"""

import numpy
from numpy import ma
from UserDict import UserDict

import threading

max_connections = 3
semaphore = threading.BoundedSemaphore(max_connections)
semaphore.acquire() # decrements the counter

semaphore.release() # increments the counter

class FilterUneven(UserDict):
    """
    """
    def __init__(self,data):
        """
	"""
	self.data = data
	return

from multiprocessing import BoundedSemaphore
from multiprocessing import Pool
from multiprocessing import Lock
from numpy import ma

data=numpy.random.random((3,3))
output=ma.ones(data.shape)*1e20


sema=BoundedSemaphore(5)

import time

p = multiprocessing.Process(target=time.sleep, args=(1000,))


def ff(i,j):
    #l.acquire()
    output[i,j,]=data.mean()+data[i,j]
    #l.release()
    return

lock = Lock()
data[i,j],output[i,j]
pool = Pool(processes=4)
pool.map(ff, [range(10),range(10)])
ff(i,j,lock)
data[i,j],output[i,j]

def f(x):
    from numpy.random import random
    I,J=d.shape
    tmp=0
    for i in range(I):
        for j in range(J):
	    tmp=tmp+data[i,j]
    tmp=tmp/(I*J)
    return x,tmp


pool = Pool(processes=4)
print pool.map(f, range(10))




def semaphore_func(sema, mutex, running):
    sema.acquire()
    mutex.acquire()
    running.value += 1
    print running.value, 'tasks are running'
    mutex.release()
    random.seed()
    time.sleep(random.random()*2)
    mutex.acquire()
    running.value -= 1
    print '%s has finished' % multiprocessing.current_process()
    mutex.release()
    sema.release()

def test_semaphore():
    sema = multiprocessing.Semaphore(3)
    mutex = multiprocessing.RLock()
    running = multiprocessing.Value('i', 0)
    processes = [
        multiprocessing.Process(target=semaphore_func,
                                args=(sema, mutex, running))
        for i in range(10)
        ]
    for p in processes:
        p.start()
    for p in processes:
        p.join()


def test_semaphore2():
    pool = multiprocessing.Pool(4)



#from multiprocessing import Pool
#pool = Pool(processes=4)

import multiprocessing
from numpy import random
import time


#def semaphore_func(sema, mutex, running):
def semaphore_func(running):
    #sema.acquire()
    #mutex.acquire()
    running.value += 1
    print running.value, 'tasks are running'
    #mutex.release()
    random.seed()
    time.sleep(random.random()*5)
    #mutex.acquire()
    running.value -= 1
    print '%s has finished' % multiprocessing.current_process().name
    #mutex.release()
    #sema.release()
    return


import multiprocessing
from numpy import random
import time

def bunda(name):
    print "value: ",name
    random.seed()
    time.sleep(random.random()*5)
    print '%s has finished' % multiprocessing.current_process().name
    return



PROCESSES = multiprocessing.cpu_count()
pool = multiprocessing.Pool(processes=PROCESSES)
map_it = pool.map(bunda, range(10))




def test_semaphore():
    sema = multiprocessing.Semaphore(3)
    #mutex = multiprocessing.RLock()
    running = multiprocessing.Value('i', 0)
    processes = [
        multiprocessing.Process(target=semaphore_func,
                                args=(sema, running))
        for i in range(10)
        ]
    for p in processes:
        p.start()
    print "Done"
    return

test_semaphore()


sema = multiprocessing.Semaphore(3)




""" Weight functions"""

import numpy as np
from numpy import ma
from numpy import pi

#from fluid.common.distance import distance

# defining some weight functions

def window_func(method='hamming'):
    """ Select the weight function
    """
    if method == 'hamming':
        winfunc = _weight_hamming

    elif method == 'hann':
        winfunc = _weight_hann

    elif method == 'blackman':
        winfunc = _weight_blackman

    elif method == 'triangular':
        winfunc = _weight_triangular

    return winfunc



# blackman
def _weight_blackman(r,l):
    """ 
    using values from wikipedia 'exact blackman'
    
    """
    w = 0.42 +  0.5*np.cos(2*pi*r/l) + 0.08*np.cos(4*pi*r/l) # fase lag-> sign change
    w[np.absolute(r)>l/2.]=0
    return w

# rectangular

# triangular
def _weight_triangular(r,l):
    """
    """
    w = ma.masked_all(r.shape)
    ind = np.abs(r)<l/2.
    ind2 = np.abs(r)>l/2.
    w[ind] = 1-np.abs(2*r[ind]/l)
    w[ind2] = 0
    return w

def _weight_hamming(r,l):
    """ Hamming weight

        w = 0.54 - 0.46*cos(2*pi*n/(N-1))
        where n is the element index of a total N elements.
        or
        w = 0.54 + 0.46*cos(2*pi*r/l),
        where r is the distance to the center of the window,
        and l is the total width of the filter window.
        hint: cos(a-b) = cos(a)cos(b)+sin(a)sin(b)
    """
    w = 0.54 + 0.46*np.cos(2*pi*r/l)
    w[np.absolute(r)>l/2.]=0
    return w

# hamming 2D
def _weight_hamming_2D(x, y, l):
    """
        check _weight_hamming
    """
    r = (x**2+y**2)**0.5
    w = 0.54 + 0.46*np.cos(2*pi*r/l)
    w[np.absolute(r)>l/2.] = 0
    return w

# hann
def _weight_hann(r,l):
    """ Hann weight 
        
        from the definition of the Hann window centered in n = (N-1)/2:
        w = 0.5*(1 + np.cos(2*pi*n/(N-1))),
        where n is the element index of a toal of N elements.
        To make it symmetrical, i.e centered in n=0, we should add a 
        phase of pi in the cosine argument. So,
        w = 0.5*(1 - np.cos(2*pi*r/l)),
        where r is the distance to the center of teh window
        and l is the total width of the window.
    """
    w=0.5*(1+np.cos(2*pi*r/l))
    w[np.absolute(r)>l/2.]=0
    return w

# hann 2D
def _weight_hann_2D(x,y,l):
    """
        Check _weight_hann 
    """
    r=(x**2+y**2)**0.5
    w=0.5*(1+np.cos(2*pi*r/l))
    w[r>l]=0
    return w

# hann 1D bandpass
def _weight_hann_band(r,l1,l2):
    """ Hann weight 
        
        from the definition of the Hann window centered in n = (N-1)/2:
        w = 0.5*(1 + np.cos(2*pi*n/(N-1))),
        where n is the element index of a toal of N elements.
        To make it symmetrical, i.e centered in n=0, we should add a 
        phase of pi in the cosine argument. So,
        w = 0.5*(1 - np.cos(2*pi*r/l)),
        where r is the distance to the center of teh window
        and l1 is the width of the first low-pass window and l2
        is the width of the second low-pass window.
        So, the band-pass is going to be for l2 > f > l1.
    """
    w= ((np.cos(pi*r/l1)**2))*((np.sin(pi*r/l2))**2)
    w[np.absolute(r)>l1/2.]=0
    w[np.absolute(r)<l2/2.]=0
    return w


# lanczos 2D
def _weight_lanczos_2D(x,y,l,cutoff):
    """ Working on
    """
    #c=cutoff
    r=(x**2+y**2)**0.5
    w=np.sinc(r/cutoff)*np.sinc(r/cutoff/l)
    w[r>3*l]=0

# ==== funcoes antigas ======================

def _weight_triangular_2D(x,y,l):
    """
    """
    r=(x**2+y**2)**0.5
    w=(l-r)/l
    w[r>l]=0
    return w



# -*- coding: utf-8 -*-
try:
    from setuptools import setup, find_packages
except ImportError:
    import distribute_setup
    distribute_setup.use_setuptools()
    from setuptools import setup, find_packages

import os
#import sys
from distutils import log

#from distutils.core import setup
# Which Extension to use?
#from distutils.extension import Extension
from Cython.Distutils.extension import Extension
from Cython.Distutils import build_ext
import numpy as np

here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, 'README.rst')).read()
NEWS = open(os.path.join(here, 'NEWS.txt')).read()

version='0.6.8'

requires = [
    'numpy>=1.1',
    'fluid>=0.2',
    'distribute>=0.6.40',
    ]

setup(
    name='maud',
    version=version,
    url='http://maud.castelao.net',
    download_url='http://pypi.python.org/packages/source/m/maud/maud-'+version+'.tar.gz',
    license='PSF',
    author='Guilherme Castelao, Bia Villas-Boas, Luiz Irber',
    author_email='guilherme@castelao.net, bia@melovillasboas.com, luiz.irber@gmail.com',
    description='Moving Average for Uneven Data',
    long_description=README + '\n\n' + NEWS,
    #zip_safe=True,
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: Python Software Foundation License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Topic :: Scientific/Engineering',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    platforms='any',
    #py_modules=['maud', 'window_func'],
    packages=find_packages(),
    #packages=['maud', 'maud.window_func'],
    install_requires=requires,
    cmdclass = {'build_ext': build_ext},
    ext_modules = [Extension("maud.cwindow_func", ["maud/window_func.pyx"]), Extension("cmaud", ["maud/maud.pyx"])],
    include_dirs = [np.get_include()],
    #ext_modules = [
    #    Extension("maud.cwindow_func", ["window_func.pyx"],
    #    #libraries=['maud'],
    #    include_dirs = [np.get_include()],
    #    #pyrex_include_dirs=['.']
    #    ),
    #    ],
    scripts=["bin/maud4nc", "bin/maud4latlonnc"],
)




# http://docs.cython.org/src/userguide/numpy_tutorial.html#numpy-tutorial

import numpy as np
from numpy import ma

cimport numpy as np
from libc.math cimport cos

DTYPE = np.float64
ctypedef np.float64_t DTYPE_t

# Hamming
def _weight_hamming(np.ndarray r, double l):
    """ Cython hamming weight

        Input:
	    r np.array
	    l double

	w = 0.54 - 0.46*cos(2*pi*n/(N-1))
	where n is the element index of a total N elements.
	or
	w = 0.54 + 0.46*cos(2*pi*r/l),
	where r is the distance to the center of the window,
	and l is the total width of the filter window.
	hint: cos(a-b) = cos(a)cos(b)+sin(a)sin(b)

    """
    #print r.dtype
    #print DTYPE
    #assert r.dtype == DTYPE
    cdef int n
    cdef int N = len(r)
    cdef double lhalf = l/2.
    cdef double scale = 2*np.pi/l
    cdef np.ndarray w = np.zeros(N, dtype=DTYPE)
    for n in range(N):
        if abs(r[n])<=lhalf:
            w[n] = 0.54 + 0.46*cos(scale*r[n])
    return w

def _weight_hann(np.ndarray r, double l):
    """ Cython Hann weight

        Input:
            r np.array
            l double

	from the definition of the Hann window centered in n = (N-1)/2:
        w = 0.5*(1 - np.cos(2*pi*n/(N-1))),
        where n is the element index of a toal of N elements.
        To make it symmetrical, i.e centered in n=0, we should add a 
        phase of pi in the cosine argument. So,
        w = 0.5*(1 + np.cos(2*pi*r/l)),
        where r is the distance to the center of teh window
        and l is the total width of the window.

    """
    #print r.dtype
    #print DTYPE
    #assert r.dtype == DTYPE
    cdef int n
    cdef int N = len(r)
    cdef double lhalf = l/2.
    cdef double scale = 2*np.pi/l
    cdef np.ndarray w = np.zeros(N, dtype=DTYPE)
    for n in range(N):
        if abs(r[n])<=lhalf:
            w[n] = 0.5*(1 + cos(scale*r[n]))
    return w


def window_func(method='hamming'):
    """ Select the weight function
    """
    if method == 'hamming':
        winfunc = _weight_hamming

    elif method == 'hann':
        winfunc = _weight_hann

    else:
        print "Filter %s is not available" % method
        return

    #elif method == 'blackman':
    #    winfunc = _weight_blackman

    #elif method == 'triangular':
    #    winfunc = _weight_triangular

    return winfunc

def window_func_scalar(method='hamming'):
    """ Select the weight function
    """
    if method == 'hamming':
        winfunc = _weight_hamming_scalar

    #elif method == 'hann':
    #    winfunc = _weight_hann_scalar

    else:
        print "Filter %s is not available" % method
        return

    #elif method == 'blackman':
    #    winfunc = _weight_blackman

    #elif method == 'triangular':
    #    winfunc = _weight_triangular

    return winfunc

# Hamming
def _weight_hamming_scalar(double r, double l):

    if abs(r) > l/2.:
        return 0

    return 0.54 + 0.46*cos(r*2*np.pi/l)
